@@ -828,6 +828,7 @@ totemPlugin::ViewerReady ()
 #ifdef TOTEM_NARROWSPACE_PLUGIN
 	/* Tell the viewer it has an href */
 	if (mHref) {
+		Dm("SetHref in ViewerReady");
 		dbus_g_proxy_call_no_reply (mViewerProxy,
 					    "SetHref",
 					    G_TYPE_STRING, mHref,
@@ -995,7 +996,6 @@ totemPlugin::RequestStream (bool aForceViewer)
 	/* Prefer filename over src */
 	if (mURLURI) {
 		requestURI = mURLURI;
-		baseURI = mSrcURI; /* FIXME: that correct? */
 	}
 #endif /* TOTEM_GMP_PLUGIN */
 
@@ -1011,11 +1011,14 @@ totemPlugin::RequestStream (bool aForceViewer)
 		requestURL = href;
 	}
 #endif
+
+       /* We should be following QTSRCDONTUSEBROWSER instead */
+	aForceViewer = true;
 #endif /* TOTEM_NARROWSPACE_PLUGIN */
 
-#ifdef TOTEM_MULLY_PLUGIN
+#if defined (TOTEM_MULLY_PLUGIN) || defined (TOTEM_CONE_PLUGIN)
 	aForceViewer = true;
-#endif /* TOTEM_MULLY_PLUGIN */
+#endif /* TOTEM_MULLY_PLUGIN || TOTEM_CONE_PLUGIN */
 
 	/* Fallback */
 	if (!requestURI)
@@ -1216,6 +1219,30 @@ totemPlugin::TickCallback (DBusGProxy *proxy,
 		}
 	}
 #endif /* TOTEM_GMP_PLUGIN */
+
+#ifdef TOTEM_NARROWSPACE_PLUGIN
+        if (!plugin->mNPObjects[ePluginScriptable].IsNull ()) {
+                NPObject *object = plugin->mNPObjects[ePluginScriptable];
+                totemNarrowSpacePlayer *scriptable = static_cast<totemNarrowSpacePlayer*>(object);
+                switch (plugin->mState) {
+		case TOTEM_STATE_PLAYING:
+		case TOTEM_STATE_PAUSED:
+			scriptable->mPluginState = totemNarrowSpacePlayer::eState_Playable;
+			break;
+		case TOTEM_STATE_STOPPED:
+			if (scriptable->mPluginState == totemNarrowSpacePlayer::eState_Playable) {
+				scriptable->mPluginState = totemNarrowSpacePlayer::eState_Complete;
+				/* The QuickTime plugin expects the duration to be the
+				 * length of the file on EOS */
+				plugin->mTime = plugin->mDuration;
+			} else
+				scriptable->mPluginState = totemNarrowSpacePlayer::eState_Waiting;
+			break;
+		default:
+			scriptable->mPluginState = totemNarrowSpacePlayer::eState_Waiting;
+		}
+	}
+#endif /* TOTEM_NARROWSPACE_PLUGIN */
 }
 
 /* static */ void
@@ -1560,7 +1587,23 @@ totemPlugin::SetURL (const char* aURL)
 
         mURLURI = g_strdup (aURL);
 
-	/* FIXME: what is the correct base for the URL param? mSrcURI or mBaseURI? */
+	/* FIXME: security checks? */
+        /* FIXMEchpe: resolve the URI here? */
+}
+
+void
+totemPlugin::SetBaseURL (const char *aBaseURL)
+{
+        g_free (mBaseURI);
+
+	/* Don't allow empty URL */
+        if (!aBaseURL || !aBaseURL[0]) {
+                mBaseURI = NULL;
+		return;
+        }
+
+        mBaseURI = g_strdup (aBaseURL);
+
 	/* FIXME: security checks? */
         /* FIXMEchpe: resolve the URI here? */
 }
@@ -1569,6 +1612,30 @@ totemPlugin::SetURL (const char* aURL)
 
 #ifdef TOTEM_NARROWSPACE_PLUGIN
 
+void
+totemPlugin::SetURL (const NPString& aURL)
+{
+        g_free (mSrcURI);
+
+	/* If |src| is empty, don't resolve the URI! Otherwise we may
+	 * try to load an (probably iframe) html document as our video stream.
+	 */
+	if (!aURL.UTF8Characters || !aURL.UTF8Length) {
+              mSrcURI = NULL;
+              return;
+        }
+
+        mSrcURI = g_strndup (aURL.UTF8Characters, aURL.UTF8Length);
+
+        UnsetStream ();
+
+        if (mAutoPlay) {
+                RequestStream (true); //FIXME
+        } else {
+                mWaitingForButtonPress = true;
+        }
+}
+
 bool
 totemPlugin::SetQtsrc (const char* aURL)
 {
@@ -2067,6 +2134,12 @@ totemPlugin::Init (NPMIMEType mimetype,
         if (filename) {
                 SetURL (filename);
         }
+
+        /* http://msdn.microsoft.com/en-us/library/dd564090(v=VS.85).aspx */
+        const char *base = (const char *) g_hash_table_lookup (args, "baseurl");
+        if (base)
+		SetBaseURL (base);
+
 #endif /* TOTEM_GMP_PLUGIN */
 
 #ifdef TOTEM_NARROWSPACE_PLUGIN
@@ -2239,6 +2312,7 @@ totemPlugin::Init (NPMIMEType mimetype,
 
 	/* Dump some disagnostics */
 	D ("mSrcURI: %s", mSrcURI ? mSrcURI : "");
+	D ("mBaseURI: %s", mBaseURI ? mBaseURI : "");
 	D ("mCache: %d", mCache);
 	D ("mControllerHidden: %d", mControllerHidden);
 	D ("mShowStatusbar: %d", mShowStatusbar);
@@ -2540,6 +2614,7 @@ totemPlugin::StreamAsFile (NPStream *stream,
 	gboolean retval = TRUE;
 	GError *error = NULL;
 	if (mIsPlaylist) {
+		Dm("Calling SetPlaylist in StreamAsFile");
 		retval = dbus_g_proxy_call (mViewerProxy,
 					    "SetPlaylist",
 					    &error,
@@ -2554,6 +2629,7 @@ totemPlugin::StreamAsFile (NPStream *stream,
 	 * completely in the cache.)
 	 */
 	else if (mBytesStreamed == 0) {
+		Dm("Calling SetLocalFile from ViewerReady");
 		retval = dbus_g_proxy_call (mViewerProxy,
 					    "SetLocalFile",
 					    &error,
@@ -2602,7 +2678,13 @@ totemPlugin::URLNotify (const char *url,
 		        NPReason reason,
 		        void *notifyData)
 {
-	D ("URLNotify URL '%s' reason %d", url ? url : "", reason);
+	const char *reasons[] = {
+		"Base (undefined)",
+		"Done",
+		"Network error",
+		"User break"
+	};
+	D ("URLNotify URL '%s' reason %d (%s)", url ? url : "", reason, reasons[reason]);
 
 	/* If we get called when we expect a stream,
 	 * it means that the stream failed.
